﻿/*
 * RV.cpp
 *
 * Created: 02.08.2023 14:39:59
 *  Author: Валера
 */ 
#define F_CPU	8000000UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include "BUT_avr.h"
#include "logic.h"


#define ledA_on		PORTD |= 1<<0
#define ledA_off	PORTD &= ~(1<<0)

#define ledB_on		PORTD |= 1<<1
#define ledB_off	PORTD &= ~(1<<1)

#define ledC_on		PORTD |= 1<<2
#define ledC_off	PORTD &= ~(1<<2)

#define relay_on	PORTD |= 1<<6
#define relay_off	PORTD &= ~(1<<6)

#define alarm_on	PORTD |= 1<<3
#define alarm_off	PORTD &= ~(1<<3)



But_avr alarm(&PIND, 7, 10, _pull_up_);//инициализация кнопки

front_port	A(&PINB, 0);	//инициализация портов и
front_port	B(&PINB, 1);	//функций определения 
front_port	C(&PINB, 2);	//положительного фронта
front_m		relay;			//инициализация маркера включения/выключения реле

u16 phA, phB, phC;			//счетчики задержки по фазам
u16 del;					//задержка включения реле
const u16 lim_ABC = 20, lim_del=2450;//лимит счетчиков задержки
bool	m_res_A, m_res_B, m_res_C;//маркеры ошибок по фазам

int main(void)
{
	m_res_A = false;
	m_res_B = false;
	m_res_C = false;
//timer0............................................
	TCCR0 |= (1<<CS00)|(1<<CS01);
	TIMSK |= 1<<TOIE0;
//ports.............................................
	DDRD |= (1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<6);	
	
	sei();
	
    while(1)
    {
		alarm.FPN_check();//проверка фронтов кнопки сброса ошибок
		relay.check_MPNF();//проверка маркера реле
		
		//сброс ошибок.................................
		if(alarm.get_MNF()){
			m_res_A = false;
			m_res_B = false;
			m_res_C = false;
		}
		
		
			
		//реле нагрузки..........................................
		if((phA < lim_ABC)&&(phB < lim_ABC)&&(phC < lim_ABC))
			relay.set_M();		//включение/выключение маркера реле	
		else
			relay.res_M();
		//.....................
		if(relay.get_MPF())		//включение задержки реле
			del = lim_del; 
		//....................	
		if(relay.get_state() && !del)
			relay_on;	//включение/выключение реле
		else
			relay_off;
			
		
		//индикация ошибок по фазам.............	
		if((phA < lim_ABC)&&(!m_res_A)){
			ledA_off;
		}else{
			ledA_on;
			m_res_A = true;
		}
		
		if((phB < lim_ABC)&&(!m_res_B)){
			ledB_off;
		}else{
			ledB_on;
			m_res_B = true;
		}
		
		if((phC < lim_ABC)&&(!m_res_C)){
			ledC_off;
		}else{
			ledC_on;
			m_res_C = true;
		}
		
		//индикация общая ошибок........................
		if(m_res_A || m_res_B || m_res_C)
			alarm_on;
		else
			alarm_off;
		
    }
	
	
	
	
}
//========================================================
ISR(TIMER0_OVF_vect){
	//Сброс счетчиков..........................
    if(A.FP_port())
		phA = 0;
	if(B.FP_port())
		phB = 0;
	if(C.FP_port())
		phC = 0;
	//инкремент счетчиков
	if(phA < lim_ABC)
		phA++;
	if(phB < lim_ABC)
		phB++;
	if(phC < lim_ABC)
		phC++;
	if(del)	
		del--;
};


